"""
    Security audit utility.
"""
from AccessControl.Permission import Permission
import OFS.FindSupport

class SecurityTweak:
    """
        Capture knowledge about one or more security tweaks applied to
        an object in the ZODB.  The captured tweaks include:

        - Local role assigments

        - Proxy roles.

        - Overridden permission mappings.

        - Executable ownership which differes from the ownership of
          the 'aq_parent'.

        - 'Owner' local roles which don't include the executable owner
          (leave out 'System Processes' for unowned).
    """
    __allow_access_to_unprotected_subobjects__ = 1
    local_roles = owners_by_role = proxy_roles = ()
    permission_overrides = {}
    new_owner = owner_mismatch = None
    capture_error = None
   
    def __init__( self, obj ):

        self.path = obj.getPhysicalPath()
        self.meta_type = obj.meta_type

        # Add exception handling per Wolfgang Strobl's suggestion.
        try:
            # Find local roles other than 'Owner'.
            self.local_roles = []
            self.owners_by_role = []
            for k, v in obj.get_local_roles():
                v = list( v )
                if 'Owner' in v:
                    self.owners_by_role.append( k )
                try:
                    v.remove( 'Owner' )
                except ValueError:
                    pass
                if v:
                    self.local_roles.append( ( k, tuple( v ) ) )

            # Find proxy roles, if any.
            self.proxy_roles = getattr( obj, '_proxy_roles', None )

            # Find overridden (non-acquired) permissions.
            po = self.permission_overrides = {}
            for p in obj.ac_inherited_permissions(1):
                name, value = p[:2]
                p = Permission( name, value, obj )
                roles = p.getRoles()
                if type( roles ) == type( () ):
                    po[ name ] = roles

            # Find "responsible" owner, compare with parent.
            owner = obj.getOwner( 1 )
            parent = getattr( obj, 'aq_parent', None )
            try:
                powner = parent is not None and parent.getOwner( 1 ) or None
            except AttributeError:
                powner = None
            self.new_owner = owner != powner and owner or None
            if owner:
                if owner[1] not in self.owners_by_role:
                    self.owner_mismatch = ( owner[1], self.owners_by_role )
                else:
                    self.owner_mismatch = None
            else:
                self.owner_mismatch = (
                    self.owners_by_role != ['System Processes']
                      and ( None, self.owners_by_role ) or None )
        except:
            import sys, traceback
            try:
                etype, value, tb = sys.exc_info()
                self.capture_error = traceback.format_exception( etype, value
                                                               , tb )
            finally:
                etype = value = tb = None

    def __nonzero__( self ):
        # Do we have anything interesting to say?
        return ( self.local_roles
              or self.proxy_roles
              or self.permission_overrides
              or self.new_owner
              or self.owner_mismatch
              or self.capture_error
               ) and 1 or 0


def allTweaks( self ):
    """
        Run over entire database, capturing any object which has
        a tweak.
    """
    found = self.ZopeFind( self, search_sub=1 )
    results = []
    add_result = results.append
    for p, obj in found:
        tweak = SecurityTweak( obj )
        if tweak:
            add_result( tweak )
    return results
